/**************************************************************************//**
 * @file     main.c
 * @version  V1.00
 * @brief    Use the timer periodic mode to generate timer interrupt every 1ms for software button debouncing.
 *
 * @copyright (C) 2019 Nuvoton Technology Corp. All rights reserved.
 *
 ****************************************************************************/
#include <stdio.h>
#include "NuMicro.h"

/*---------------------------------------------------------------------------------------------------------*/
/* Define                                                                                                  */
/*---------------------------------------------------------------------------------------------------------*/
#define DEBOUNCE_TIME 50            //Press and release button settling time ( ms )
#define BUTTON1 PC14                //Button input port    
#define LED PB14                    //Indicator light used to indicate that a button is pressed(debounced)
#define TEST_PIN PB15               //Used to measure button state without debouncing

/*---------------------------------------------------------------------------------------------------------*/
/* Global variables                                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
uint8_t bTimer_1ms;                 //Timer 1ms flag
uint8_t g_ButtonState;              //BUTTON1's current state
uint8_t g_ButtonStateOld;           //BUTTON1's previous state
uint8_t g_ButtonStateDebounced = 1; //BUTTON1's de-bounced state
uint32_t g_ButtonDebounceCnt = 0;
/*---------------------------------------------------------------------------------------------------------*/
/* Functions                                                                                               */
/*---------------------------------------------------------------------------------------------------------*/
uint8_t GPIO_Debounce(void);

void TMR0_IRQHandler(void)
{
    /* Set 1ms flag */
    bTimer_1ms = 1;
    /* clear timer interrupt flag */
    TIMER_ClearIntFlag(TIMER0);
}

void SYS_Init(void)
{
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable HIRC */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRCEN_Msk);

    /* Waiting for HIRC clock ready */
    CLK_WaitClockReady(CLK_STATUS_HIRCSTB_Msk);

    /* Switch HCLK clock source to HIRC */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HIRC, CLK_CLKDIV0_HCLK(1));

    /* Set both PCLK0 and PCLK1 as HCLK/2 */
    CLK->PCLKDIV = (CLK_PCLKDIV_APB0DIV_DIV2 | CLK_PCLKDIV_APB1DIV_DIV2);

    /* Switch UART0 clock source to HIRC */
    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UART0SEL_HIRC, CLK_CLKDIV0_UART0(1));

    /* Enable UART peripheral clock */
    CLK_EnableModuleClock(UART0_MODULE);

    /* Enable IP clock */
    CLK_EnableModuleClock(TMR0_MODULE);

    /* Select IP clock source */
    CLK_SetModuleClock(TMR0_MODULE, CLK_CLKSEL1_TMR0SEL_HIRC, 0);

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */
    SystemCoreClockUpdate();

    /*----------------------------------------------------------------------*/
    /* Init I/O Multi-function                                              */
    /*----------------------------------------------------------------------*/
    /* Set GPB multi-function pins for UART0 RXD and TXD */
    SYS->GPB_MFPH = (SYS->GPB_MFPH & ~(SYS_GPB_MFPH_PB12MFP_Msk | SYS_GPB_MFPH_PB13MFP_Msk)) |
                    (SYS_GPB_MFPH_PB12MFP_UART0_RXD | SYS_GPB_MFPH_PB13MFP_UART0_TXD);

    /* Lock protected registers */
    SYS_LockReg();
}

/*----------------------------------------------------------------------*/
/* Init UART0                                                           */
/*----------------------------------------------------------------------*/
void UART0_Init(void)
{
    /* Reset UART0 */
    SYS_ResetModule(UART0_RST);

    /* Configure UART0 and set UART0 baud rate */
    UART_Open(UART0, 115200);
}

int main(void)
{
    /* Init System, IP clock and multi-function I/O. */
    SYS_Init();

    /* Init UART0 for printf */
    UART0_Init();

    printf("\nThis sample code use timer to generate interrupt every 1 ms for software debouncing test \n");

    /* Set timer frequency to 1000 Hz */
    TIMER_Open(TIMER0, TIMER_PERIODIC_MODE, 1000);

    /* Enable timer interrupt */
    TIMER_EnableInt(TIMER0);
    NVIC_EnableIRQ(TMR0_IRQn);

    /* Start Timer 0 */
    TIMER_Start(TIMER0);

    /* Set PC4 to QUASI mode for BUTTON1 debouncing test*/
    GPIO_SetMode(PC, BIT14, GPIO_MODE_QUASI);

    /* Set PB14 and PB15 to output mode for BUTTON1's debouncing test*/
    GPIO_SetMode(PB, BIT14, GPIO_MODE_OUTPUT);//LED
    GPIO_SetMode(PB, BIT15, GPIO_MODE_OUTPUT);//TEST_PIN

    while (1)
    {
        /* Output BUTTON1 state to TESE_PIN */
        TEST_PIN = BUTTON1;

        /* Check the BUTTON1 state every 1ms */
        if (bTimer_1ms)
        {
            /* Clear 1ms flag */
            bTimer_1ms = 0;
            /* Output debounced BUTTON1 state to LED */
            LED = GPIO_Debounce();
        }
    }
}


uint8_t GPIO_Debounce(void)
{
    /* Read the BUTTON1 state */
    g_ButtonState = BUTTON1;

    /* Increase g_ButtonState if the previous state is the same as the current state */
    if (g_ButtonStateOld == g_ButtonState)
    {
        g_ButtonDebounceCnt++;

        /*The state is held longer than the button settling time,
          so this state can be treated as the correct state of the button. */
        if (g_ButtonDebounceCnt >= DEBOUNCE_TIME)
        {
            g_ButtonStateDebounced = g_ButtonState;
        }

        /* Clear the count to zero if the current state of the button is different from the previous state */
    }
    else
    {
        g_ButtonDebounceCnt = 0;
    }

    g_ButtonStateOld = g_ButtonState;

    /* Return the correct state of the button */
    return  g_ButtonStateDebounced;
}


